/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Save File to.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


var savefileto =
{
    get mode()   { return document.getElementById("mode"); },
    get radio()  { return document.getElementById("savefileto-radio"); },
    get button() { return document.getElementById("savefileto-button"); },
    
    get selected()      { return this.radio.selected; },
    set selected(value) { this.mode.selectedItem = this.radio; },
    
    onLoad: function()
    {
        Components.utils.import("resource://gre/modules/DownloadPaths.jsm", this);
        
        Components.utils.import("resource://savefileto/common.jsm", this);
        Components.utils.import("resource://savefileto/DataSource.jsm", this);
        Components.utils.import("resource://savefileto/savefileto.jsm", this);
        
        var self = this;
        
        var enabled = this.savefileto.prefs.fileEnabled;
        document.getElementById("savefileto-box").hidden = (! enabled);
        
        if (! enabled)
            return;
        
        // show normalBox instead of basicBox
        // idea copied from DownThemAll!
        // implementation is slightly different, so the actual result varies depending whose load handler is run first
        if (! document.getElementById("basicBox").collapsed)
        {
            document.getElementById("open").parentNode.collapsed = true;
            document.getElementById("basicBox").collapsed = true;
            document.getElementById("normalBox").collapsed = false;
        }
        
        this.filename = document.getElementById("location").getAttribute("realname");
        this.prompt = false;
        
        // default to most recent folder if there is one
        this.folder = this.savefileto.getMostRecentFolder("file");
        this.button.label = (this.folder ? this.folder.label : "");
        
        // create cascade menu
        var commandCallback = function(e, folder) { self.onCommand(e, folder); }
        var cascadeMenu = this.savefileto.createCascadeMenu("file", commandCallback);
        cascadeMenu.attach(this.button);
        
        // select Save File to when button popup opened
        var menuPopup = this.button.firstChild;
        menuPopup.addEventListener("popupshowing", function() { self.selected = true; }, false);
        
        // pre and post mode select listeners
        document.addEventListener("select", function(e) { self.onBeforeModeSelect(e); }, true);
        //this.mode.addEventListener("select", function(e) { dump("mode select\n"); }, false);
        this.radio.addEventListener("RadioStateChange", function(e) { self.onAfterModeSelect(e); }, false);
        this.oldSelected = false;
        
        // dialog accept listener
        var oldValue = document.documentElement.getAttribute("ondialogaccept");
        var newValue = "if (savefileto.selected) return savefileto.onDialogAccept(); else { " + oldValue + " }";
        document.documentElement.setAttribute("ondialogaccept", newValue);
    },
    
    onBeforeModeSelect: function(e)
    {
        if (e.target != this.mode)
            return;
        
        var selected = this.selected;
        if (selected || (selected == this.oldSelected))
            return;
        
        this._update();
    },
    
    onAfterModeSelect: function(e)
    {
        var selected = this.selected;
        if ((! selected) && (selected == this.oldSelected))
            return;
        
        this._update();
    },
    
    onCommand: function(e, folder)
    {
        this.folder = folder;
        this.prompt = e.ctrlKey;
        this.button.label = this.folder.pathLabel;
        
        this._update();
        
        document.documentElement.acceptDialog();
    },
    
    // http://mxr.mozilla.org/firefox/source/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in#850
    // http://mxr.mozilla.org/firefox/source/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in#506
    
    onDialogAccept: function()
    {
        var self = this;
        var launcher = dialog.mLauncher;
        var prompt = (this.prompt || this.savefileto.prefs.prompt);
        var folder = this.folder;
        var file = null;
        
        if (! prompt)
        {
            file = folder.file.clone();
            file.append(this.filename);
            file = this.DownloadPaths.createNiceUniqueFile(file);
        }
        
        // clean up (from nsHelperAppDlg.js.in)
        launcher.setWebProgressListener(null);
        dialog.mDialog.dialog = null;
        
        // don't start downloading until dialog has closed
        var _saveToDisk = function() { self.callLater(0, self, self._saveToDisk, [launcher, prompt, folder, file]); };
        window.addEventListener("unload", _saveToDisk, false);
        
        return true;
    },
    
    _saveToDisk: function(launcher, prompt, folder, file)
    {
        var prefsBackup = this.savefileto.getDownloadPrefs();
        
        try
        {
            var prefs = {useDownloadDir: (! prompt), folderList: 2, dir: folder.file, lastDir: folder.file};
            this.savefileto.setDownloadPrefs(prefs);
            
            // saveToDisk() throws if its prompt dialog is cancelled - don't bother catching
            launcher.saveToDisk((prompt ? null : file), false);
            this.savefileto.addToRecent("file", folder);
        }
        finally
        {
            // restore original browser download preferences
            this.savefileto.setDownloadPrefs(prefsBackup);
        }
    },
    
    _update: function()
    {
        var selected = this.selected;
        var hasFolder = this.folder;
        
        this.oldSelected = selected;
        
        this.button.style.MozUserFocus = (selected ? "normal" : "ignore");
        document.getElementById("openHandler").style.MozUserFocus = (selected ? "ignore" : "normal");
        document.getElementById("chooseButton").style.MozUserFocus = (selected ? "ignore" : "normal");
        document.getElementById("rememberChoice").disabled = selected;
        document.documentElement.getButton("accept").disabled = (selected && (! hasFolder));
    }
}


window.addEventListener("load", function() { savefileto.onLoad(); }, false);
